#!/usr/bin/python -tt
#
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: yumexCallbacks.py,v 1.2 2006/01/24 14:26:58 timlau Exp $

# Yum Extender callback handlers
import sys
import os
#import locale
#import time # test purposes only
import re
import gtk

import rpm

#import yum
#import yum.Errors as Errors
from yumgui.callbacks import *

from urlgrabber.progress import *

from yumex.yumexi18n import _
from yumex import *
from yumgui import *

          
class yumexDownloadProgress( BaseMeter ):
    """ Customized version of progress.BaseMeter class """
    def __init__( self, console, progress=None ):
        BaseMeter.__init__( self )
        self.console = console 
        self.progress = progress
        self.totSize = ""
        self.downloadTotal = 0.0
        self.downloadNow = 0.0
     
    def clearStats( self ):
        self.downloadTotal = 0.0
        self.downloadNow = 0.0
         
    def setStats( self, total ):
        self.downloadTotal = total
        self.downloadNow = 0.0
          
    def update( self, amount_read, now=None ):
        doGtkEvents()
        BaseMeter.update( self, amount_read, now )           

    def _do_start( self, now=None ):
        if self.progress != None:
            self.progress.show() 
            if self.text is not None:
                text = self.text
            else:
                text = self.basename
            self.progress.set_progress( 0, " " )
            self.progress.set_extraLabel( text )
            self.progress.set_etaLabel( "" )
            if not self.size is None:
                self.totSize = format_number( self.size )

    def _do_update( self, amount_read, now=None ):
        etime = self.re.elapsed_time()
        fetime = format_time( etime )
        fread = format_number( amount_read )
        if self.downloadTotal > 0:
            now = self.downloadNow + amount_read
            self.progress.setTotal( now, self.downloadTotal )
         #self.size = None
        if self.text is not None:
            text = self.text
        else:
            text = self.basename
          
        if self.size is None:
              #out = '%-60s : %5sB %s ' % ( text, fread, fetime )
              out = '%s' % ( text )
              eta = '%5sB %s' % ( fread, fetime )
               
              frac = 0.0
        else:
              rtime = self.re.remaining_time()
              frtime = format_time( rtime )
              frac = self.re.fraction_read()
              out = '%s' % ( text )
              eta = '%5sB/%5sB %8s ETA' % ( fread, self.totSize, frtime )
              #out = '%-60s : %5sB/%5sB %8s ETA ' % ( text, fread, self.totSize, frtime )
         #self.console.write( out )
        if self.progress != None:
            percent  = "%3i%%" % int( frac*100 )
            self.progress.set_progress( frac, percent )
            self.progress.set_extraLabel( out )
            self.progress.set_etaLabel( eta )
              

    def _do_end( self, amount_read, now=None ):
        if self.downloadTotal> 0:
            self.downloadNow += amount_read  
        total_time = format_time( self.re.elapsed_time() )
        total_size = format_number( amount_read )
        if self.text is not None:
              text = self.text
        else:
              text = self.basename
        if self.progress != None:
            self.progress.set_progress( 1.0, "%3i%%" % 100 )
            self.progress.set_extraLabel( text )
            self.progress.set_etaLabel( "" )

class yumexRPMCallback( RPMInstallCallback ):

    def __init__( self, output=1, console=None, progress=None ):
        RPMInstallCallback.__init__( self, output )
        self.console = console
        self.progress = progress
    
    def _makedone( self ):
        l = len( str( self.total_actions ) )
        size = "%s.%s" % ( l, l )
        fmt_done = "[%" + size + "s/%" + size + "s]"
        done = fmt_done % ( self.total_installed + self.total_removed, 
                           self.total_actions )
        return done       


    def instProgress( self, what, bytes, total, h, user ):
        hdr, rpmloc = h
        process =  self._process( hdr )
        out = "%s : %s" % ( process, hdr['name'] )
        self.progress.set_extraLabel( out )
        eta = self._makedone()
        self.progress.set_etaLabel( eta )
        percent = self._percent( total, bytes )
        now = float( self.total_installed-1 + self.total_removed )+float( percent )/100
        self.progress.setTotal( now, self.total_actions )
        self.progress.set_progress( float( percent )/100, "%3i%%" % percent )

    def unInstStart( self, what, bytes, total, h, user ):
        if h not in self.installed_pkg_names:
            process = _( "Removing" )
        else:
            process = _( "Cleanup" )
        #self.total_removed += 1
        out = "%s : %s" % ( process, h )
        self.progress.set_extraLabel( out )
        eta = self._makedone()
        self.progress.set_etaLabel( eta )

    def unInstProgress( self, what, bytes, total, h, user ):
        percent = self._percent( total, bytes )
        now = float( self.total_installed + self.total_removed )+float( percent )/100
        self.progress.setTotal( now, self.total_actions )
        self.progress.set_progress( float( percent )/100, "%3i%%" % percent )
        
        

class CacheProgressCallback:

     '''
     The class handles text output callbacks during metadata cache updates.
     '''
     
     def __init__( self, log, errorlog, filelog=None, console=None, progress=None ):
          self.log = log
          self.errorlog = errorlog
          self.filelog = filelog
          self.console = console
          self.progress = progress

     def log( self, level, message ):
          self.log( level, message )

     def errorlog( self, level, message ):
          if self.errorlog:
                self.errorlog( level, message )

     def filelog( self, level, message ):
          if self.filelog:
                self.filelog( level, message )

     def progressbar( self, current, total, name=None ):
        if current == 1:
            #self.console.write_line( "Loading metadata from : %s \n" % name )
            if self.progress != None:
                self.progress.set_extraLabel( _( "Processing metadata from : %s" ) % name )
                self.progress.set_etaLabel( "" )
                
        else:
            if self.progress != None:
                if current % 10 == 0 or current == total:
                    if total != 0:
                         frac = float( current )/float( total )
                    else:
                         frac = 0.0                    
                    txt = "%i/%i" % ( current, total )
                    self.progress.set_progress( frac, txt )                       
#                while gtk.events_pending():      # process gtk events
#                    gtk.main_iteration()



# This class was copied from /usr/share/yum-cli/output.py    
# The output string is made translateable

class DepSolveProgressCallBack:
    """provides text output callback functions for Dependency Solver callback"""
    
    def __init__( self, log, errorlog ):
        """requires yum-cli log and errorlog functions as arguments"""
        self.log = log
        self.errorlog = errorlog
        self.loops = 0
  
            
    def pkgAdded( self, pkgtup, mode ):
        modedict = { 'i': 'installed', 
                     'u': 'updated', 
                     'o': 'obsoleted', 
                     'e': 'erased'}
        ( n, a, e, v, r ) = pkgtup
        modeterm = modedict[mode]
        msg = _( '---> Package %s.%s %s:%s-%s set to be %s' ) % ( n, a, e, v, r, modeterm )
        self.log( 2, msg )
        
    def start( self ):
        self.loops += 1
        
    def tscheck( self ):
        self.log( 2, _( '--> Running transaction check' ) )
        
    def restartLoop( self ):
        self.loops += 1
        self.log( 2, _( '--> Restarting Dependency Resolution with new changes.' ) )
        self.log( 3, '---> Loop Number: %d' % self.loops )
    
    def end( self ):
        self.log( 2, _( '--> Finished Dependency Resolution' ) )

    
    def procReq( self, name, formatted_req ):
        msg = _( '--> Processing Dependency: %s for package: %s' ) % ( formatted_req, name )
        self.log( 2, msg )
        
    
    def unresolved( self, msg ):
        self.log( 2, _( '--> Unresolved Dependency: %s' ) % msg )

    
    def procConflict( self, name, confname ):
        self.log( 2, _( '--> Processing Conflict: %s conflicts %s' ) % ( name, confname ) )

    def transactionPopulation( self ):
        self.log( 2, _( '--> Populating transaction set with selected packages. Please wait.' ) )
    
    def downloadHeader( self, name ):
        msg = _( '---> Downloading header for %s to pack into transaction set.' ) % name
        self.log( 2, msg )
