#!/usr/bin/python -tt
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: yumexDialogs.py,v 1.20 2006/02/25 19:13:57 timlau Exp $

"""Yum Extender Dialogs"""

import sys, os, gobject, glob, shutil, datetime
import gobject
import gtk
import gtk.glade
import ConfigParser
import pygtk
import pango
import re
from yumex.yumexUtils import *
from yumex.yumexi18n import _
import yumgui


class Preferences:
    def __init__( self, xmlfile ):
        self.Tree=gtk.glade.XML ( xmlfile, 'pref' )
        self.conf = read_yumex_conf()
        self.setup_gui()
        gtk.main()
                     

    def setup_gui( self ):
        self.win = self.Tree.get_widget( 'pref' )
        self.win.connect( "destroy", self.quit )
        self.win.connect( "delete_event", self.quit )
        self.bOK = self.Tree.get_widget( "prefOK" )
        self.bOK.connect( "clicked", self.click, "OK" )
        self.bCancel = self.Tree.get_widget( "prefCancel" )
        self.bCancel.connect( "clicked", self.click, "CANCEL" )
        self.autoRefresh = self.Tree.get_widget( "prefAutoRefresh" )
        self.autoRefresh.set_active( self.conf.autorefresh )
        self.debugMode = self.Tree.get_widget( "prefDebug" )
        self.debugMode.set_active( self.conf.debug )
        self.fileList = self.Tree.get_widget( "prefFilelists" )
        self.fileList.set_active( self.conf.filelists )
        self.autoCleanup = self.Tree.get_widget( "prefCleanup" )
        self.autoCleanup.set_active( self.conf.autocleanup )
        self.safemode = self.Tree.get_widget( "prefSafeMode" )
        self.safemode.set_active( self.conf.processsafemode)
        self.MirrorDefault = self.Tree.get_widget( "prefMirrorDefault" )
        self.MirrorFastest = self.Tree.get_widget( "prefMirrorFastest" )
        self.MirrorBest = self.Tree.get_widget( "prefMirrorBest" )
        self.get_mirror_detection()
        self.excList = self.Tree.get_widget( "excView" )
        self.excBuffer = self.excList.get_buffer()
        self.get_excludes()
        self.pluginView = pluginView( self.Tree.get_widget( "prefPluginView" ) )
        self.proxy = self.Tree.get_widget( "prefProxy" )
        self.proxy.connect( "toggled", self.on_proxy )
        self.proxySrv = self.Tree.get_widget( "prefProxySrv" )
        self.proxyBox = self.Tree.get_widget( "prefProxyBox" )
        proxy = self.conf.proxy
        if proxy:
            self.proxyBox.show_all()
            self.proxy.set_active( True )
            self.proxySrv.set_text( proxy )
        else:
            self.proxyBox.hide_all()
            self.proxy.set_active( False )
            self.proxySrv.set_text( "" )
            
        self.win.show() 

    def get_excludes( self ):
        for ex in self.conf.exclude:
            self.addline( ex )
                    
    def set_excludes( self ):
        self.conf.exclude = self.get_lines()
        
    def get_lines( self ):
        start = self.excBuffer.get_start_iter()
        end = self.excBuffer.get_end_iter()
        txt = self.excBuffer.get_text( start, end )
        return txt.split( '\n' )
                
    def addline( self, txt ):
        end = self.excBuffer.get_end_iter()
        self.excBuffer.insert( end, "%s\n" % txt )

        
    def get_mirror_detection( self ):
        m = self.conf.mirrordetection
        if m != None: 
           m = m.lower()
        if m == None or m == 'best':
            self.MirrorBest.set_active( True )
        elif m == 'fastest':
            self.MirrorFastest.set_active( True )
        else:
            self.MirrorDefault.set_active( True )
            
    def set_mirror_detection( self ):
        if self.MirrorBest.get_active():
            m = 'best'
        elif self.MirrorFastest.get_active():
            m = 'fastest'
        else:
            m = 'default'
        self.conf.mirrordetection = m
        
    def on_proxy( self, widget ):        
        if self.proxy.get_active():
            self.proxyBox.show_all()
        else:
            self.proxyBox.hide_all()
            
        
        
    
    def update_conf( self ):
        self.conf.autorefresh = self.autoRefresh.get_active() 
        self.conf.debug = self.debugMode.get_active()
        self.conf.filelists = self.fileList.get_active() 
        self.conf.autocleanup = self.autoCleanup.get_active() 
        self.conf.processsafemode = self.safemode.get_active() 
        
        self.set_mirror_detection()
        self.set_excludes()
        if self.proxy.get_active():
            proxy = self.proxySrv.get_text()
            self.conf.proxy = proxy
        else:
            self.conf.proxy = ""
            

        
    def save( self ):       
        self.conf.cfg.write( open( '/etc/yumex.conf', "wt" ) )
        self.pluginView.save()

    def quit( self, w=None, event=None ):
        self.win.hide()
        self.win.destroy()
        gtk.main_quit()

    def click( self, button, key ):
        if key == "OK":
            self.update_conf()
            self.save()
            self.quit()
        elif key == "CANCEL":
            self.quit()
        elif key == "PROXY":
            self.proxyState = self.useProxy.get_active()
            if self.proxyState:
                self.boxProxy.show()     
            else:
                self.boxProxy.hide()     
            

class ProgressTotal:
    def __init__( self, widget ): 
        self.progress = widget
        self.steps = []
        self.nowProgres = 0.0
        self.clear()
    
    def setup( self, steps ):
        self.steps = steps
        self.numSteps=len( steps )
        self.currentStep=0 
        self.nowProgress = 0.0
        self.clear()

    def next( self ):
        now = 0.0
        if self.currentStep < self.numSteps:
            self.currentStep += 1
            for i in range( 0, self.currentStep ):
                now += self.steps[i]
                self.nowProgress = now
                self.setAbsProgress( now )
            return True
        else:
            return False

    def _percent( self, total, now ):
       if total == 0:
           percent = 0
       else:
           percent = ( now*100L )/total
       return percent
       
    def clear( self ):
        self.progress.set_fraction( 0 )
        self.progress.set_text( " " )
            
    def setProgress( self, now, total, prefix=None ):
        relStep = float( now )/float( total )
        curStep =self.steps[self.currentStep]
        absStep = curStep * relStep
        absProgress = self.nowProgress + absStep
        self.setAbsProgress( absProgress, prefix )
        return False
        
    def setAbsProgress( self, now, prefix=None ):
        procent = long( self._percent( 1, now ) )
        self.progress.set_fraction( now )
        if prefix:
            text = "%s : %3i%%" % ( prefix, procent )
        else:
            text = "%3i%%" % procent
        self.progress.set_text( text )
        
        
        
class ProgressDialog:
    """ Progress DialogClass """
   
    def __init__( self, tree, gfxPath, notebook, set_page_func ):
        self.Tree=tree
        self.set_page_func = set_page_func
        self.notebook = notebook
        self.box =self.Tree.get_widget( "progressBox" )
        self.mainLabel = self.Tree.get_widget( "progressMainLabel" )
        self.mainLabel.set_text( "" )
        self.subLabel = self.Tree.get_widget( "progressSubLabel" )
        self.subLabel.set_text( "" )
        self.extraLabel = self.Tree.get_widget( "progressExtraLabel" )
        self.extraLabel.set_text( "" )
        self.etaLabel = self.Tree.get_widget( "progressETALabel" )
        self.etaLabel.set_text( "" )
        self.progress = self.Tree.get_widget( "progressBar" )
        self.totalWidget = self.Tree.get_widget( "totalProgressBar" )
        self.total = ProgressTotal( self.totalWidget )
        self.progress.set_fraction( 0 )
        self.progress.set_text( " " )
            
    def show( self ):
        self.box.show()
        self.set_page_func( 'output' )
        
    def hide( self, clean=False ):
        self.box.hide()
        if clean:
            self.mainLabel.set_text( "" )
            self.subLabel.set_text( "" )
            self.extraLabel.set_text( "" )
            self.progress.set_fraction( 0 )
            self.progress.set_text( " " )
        
    def setTotal( self, now, total ):
        self.total.setProgress( now, total )
        yumgui.doGtkEvents()
    
    def set_progress( self, frac, text=None ): 
        if frac >= 0 and frac <= 1:
            self.progress.set_fraction( frac )
        else:
            self.progress.set_fraction( 0 )
        if text != None:
            self.progress.set_text( text )
        yumgui.doGtkEvents()
       
    def set_mainLabel( self, text ):
        self.mainLabel.set_markup( "<span size=\"large\"><b>%s</b></span>" % text )
        self.subLabel.set_text( "" )
        self.extraLabel.set_text( "" )
           
    def set_subLabel( self, text ):
        self.subLabel.set_markup( "%s" % text )
        self.extraLabel.set_text( "" )
        self.etaLabel.set_text( "" )

    def set_extraLabel( self, text ):
        self.extraLabel.set_markup( "<span size=\"small\">%s</span>" % text )
        self.etaLabel.set_text( "" )
        
    def set_etaLabel( self, text ):
        self.etaLabel.set_markup( "<span size=\"small\">%s</span>" % text )
                             


class EditRepo:
    def __init__( self, xmlfile, thisrepo, isNew= False ):
        # Setup Glade Widgets
        self.xml = gtk.glade.XML( xmlfile, "repowin" )
        self.window = self.xml.get_widget( "repowin" )
        self.ok = self.xml.get_widget( "repoOK" )        
        self.cancel = self.xml.get_widget( "repoCancel" )        
        self.optEnabled = self.xml.get_widget( "repoEnabled" )
        self.optGPG = self.xml.get_widget( "repoGPG" )    
        self.optMirrorList=self.xml.get_widget( "repoMirrorlist" )        
        self.optBaseurl = self.xml.get_widget( "repoBaseurl" )        
        self.title = self.xml.get_widget( "repoTitle" )    
        self.name = self.xml.get_widget( "repoName" )     
        self.GPGKey = self.xml.get_widget( "repoGPGKey" )              
        self.URL = self.xml.get_widget( "repoURL" )        
        
        # Other Member

        self.notSaved = False
        
        # Setup Event Handlers    
        self.window.connect( "delete_event", self.on_Delete )
        self.window.connect( "frame_event", self.on_Delete )
        # Buttons Events
        self.ok.connect( "clicked", self.on_OK )
        self.cancel.connect( "clicked", self.on_Cancel )
        self.repo = Repository()        
        self.window.present()
        if not isNew:
            self.repo.set_active( thisrepo )
            self.update_fields()    
        else:
            self.new_repo()

    def show( self ):
        #print "EditRepo: show"
        self.window.present()
    
    def hide( self ):
        #print "EditRepo: hide"
        self.window.hide()
        
    def update_fields( self ):
        self.optEnabled.set_active( self.repo.get_flag( "enabled",True ) )
        self.optGPG.set_active( self.repo.get_flag( "gpgcheck" ) )
        if self.repo.isBaseUrl():
            self.optMirrorList.set_active( False )
            self.optBaseurl.set_active( True )
            urls = self.repo.get_option( "baseurl" )
        else:
            self.optMirrorList.set_active( True )
            self.optBaseurl.set_active( False )
            urls = self.repo.get_option( "mirrorlist" )
        self.orgTitle = self.repo.activeRepo
        self.title.set_text( self.orgTitle )
        self.name.set_text( self.repo.get_option( "name" ) )
        gpgkey = self.repo.get_option( "gpgkey" )
        if gpgkey != None:
             self.GPGKey.set_text( gpgkey )
        else:
             self.GPGKey.set_text( "" )
             
        self.writelines( self.URL, urls )
        self.notSaved = True;
        
    def writelines( self, view, lines ):
        buf = view.get_buffer()
        buf.set_text( "" )
        start = buf.get_start_iter()
        buf.insert( start, lines )
                
    def on_repomenu_activate( self, repo ):
        self.checkSave()
        self.repo.set_active( repo )
        self.update_fields()    
    
    def on_OK( self, key ):
        self.checkSave()
        self.window.hide()
    
    def on_Cancel( self, key ):
        self.window.hide()
        
    def on_Delete( self, event, signal ):
        self.checkSave()
        self.hide()
        event.emit_stop_by_name( "delete_event" )
        return True

    def new_repo( self ):
        title=input_box( _( "New Repository" ), _( "_New repository tag name:" ) )
        #print title
        if title != None and title.strip() != "":
            if self.repo.create_new( title, _( "New Repository" ), "baseurl" ):
                self.update_fields()    
            else:
               err_box(self.window,_( "New Repository" ),_("Repository Creation Failed" ) )

        
    def readlines( self ):
        lines = []
        buf = self.URL.get_buffer()
        start = buf.get_start_iter()
        end = buf.get_end_iter()
        lines = buf.get_text( start, end )
        return lines
        
        
    def update_repo( self ):
        self.repo.set_option( "name", self.name.get_text() )
        lines = self.readlines()
        if self.optBaseurl.get_active():                    # Check for baseurl
            if not self.repo.isBaseUrl():                # check mirrorlist -> baseurl    
                self.repo.remove_option( "mirrorlist" )    #        remove mirrorlist
            self.repo.set_option( "baseurl", lines )        #    add baseurl    
        else:                                                        # Mirrorlist, only one line    
            if self.repo.isBaseUrl():                            # check baseurl -> mirrorlist
                self.repo.remove_option( "baseurl" )        #        remove baseurl
            line = lines.split( "\n" )[0]                    # Only use first url.
            self.repo.set_option( "mirrorlist", line )    # add mirrorlist        
        self.repo.set_flag( "enabled", self.optEnabled.get_active() )
        self.repo.set_flag( "gpgcheck", self.optGPG.get_active() )
        gpgkey = self.GPGKey.get_text()
        if gpgkey != "":
             self.repo.set_option( "gpgkey", gpgkey )

    def checkSave( self ):
        if self.isChanged():
            if self.notSaved:
                rc=info_box(self.window,_('Save Changes'),_( "Do you want to save changes ?" ),cancel=True )
                if rc:                    
                    self.update_repo()
                    self.repo.save()

        
    def isChanged( self ):
        urls = self.readlines()
        if self.name.get_text() != self.repo.get_option( "name" ):
            Changed = True
        elif  self.optBaseurl.get_active() and urls != self.repo.get_option( "baseurl" ):
            Changed = True
        elif  self.optMirrorList.get_active() and urls != self.repo.get_option( "mirrorlist" ):
            Changed = True
        elif self.optBaseurl.get_active() != self.repo.isBaseUrl():
            Changed = True
        elif self.optEnabled.get_active() != self.repo.get_flag( "enabled" ):
            Changed = True
        elif self.optGPG.get_active() != self.repo.get_flag( "gpgcheck" ):
            Changed = True
        elif self.GPGKey.get_text() != self.repo.get_option( "gpgkey" ):
            if self.GPGKey.get_text() <> "":
                 Changed = True
            else:
                 Changed = False
        else:
            Changed = False
        return Changed            
        

class FileChooser( gtk.FileSelection ):
    def __init__( self, modal=True, multiple=True, pattern="*.*" ):
        gtk.FileSelection.__init__( self )
        self.multiple = multiple
        self.connect( "destroy", self.quit )
        self.connect( "delete_event", self.quit )
        if modal:
            self.set_modal( True )
        self.cancel_button.connect( 'clicked', self.quit )
        self.ok_button.connect( 'clicked', self.ok_cb )
        if multiple:
            self.set_select_multiple( True )
            ## self.hide_fileop_buttons()
        self.complete( pattern )
        self.ret = None
    def quit( self, *args ):
        self.hide()
        self.destroy()
        gtk.main_quit()
    def ok_cb( self, b ):
        if self.multiple:
            self.ret = self.get_selections()
        else:
            self.ret = self.get_filename()
        self.quit()


class EntryDialog( gtk.Dialog ):
    def __init__( self, message="", default_text='', modal= True ):
        gtk.Dialog.__init__( self )
        self.connect( "destroy", self.quit )
        self.connect( "delete_event", self.quit )
        if modal:
            self.set_modal( True )
        box = gtk.VBox( spacing=6 )
        box.set_border_width( 5 )
        self.set_border_width( 5 )
        self.vbox.set_spacing( 2 )
        self.vbox.pack_start( box )
        box.show()
        if message:
            label = gtk.Label( message )
            label.set_alignment( 0.0, 0.5 )
            label.set_use_underline( True )
            box.pack_start( label )
            label.show()
        self.set_has_separator( False )
        self.entry = gtk.Entry()
        self.entry.set_text( default_text )
        box.pack_start( self.entry )
        self.entry.show()
        self.entry.grab_focus()
        label.set_mnemonic_widget( self.entry )
        button = gtk.Button( "Cancel", "gtk-cancel", True )
        button.connect( "clicked", self.quit )
        button.set_flags( gtk.CAN_DEFAULT )
        self.action_area.pack_start( button )
        button.show()
        button = gtk.Button( "OK", "gtk-ok", True )
        button.connect( "clicked", self.click )
        button.set_flags( gtk.CAN_DEFAULT )
        self.action_area.pack_start( button )
        button.show()
        button.grab_default()
        self.ret = None
    def quit( self, w=None, event=None ):
        self.hide()
        self.destroy()
        gtk.main_quit()
        
    def click( self, button ):
        self.ret = self.entry.get_text()
        self.quit()
        
class ErrorDialog:
    def __init__( self, parent, title, text, longtext, modal ):
        self.MAIN_PATH = os.path.abspath( os.path.dirname( sys.argv[0] ) );
        self.GLADE_FILE = self.MAIN_PATH+'/yumex.glade'  
        self.xml = gtk.glade.XML( self.GLADE_FILE, "errDialog" )
        self.dialog = self.xml.get_widget( "errDialog" )
        self.parent = parent
        self.dialog.set_transient_for( parent )
        #self.dialog.set_icon_name( 'gtk-dialog-error' )
        self.dialog.set_title( title )
        self.text = self.xml.get_widget( "errText" )
        self.longtext = self.xml.get_widget( "errTextView" )
        self.style_err = gtk.TextTag( "error" ) 
        self.style_err.set_property( "style", pango.STYLE_ITALIC )
        self.style_err.set_property( "foreground", "red" )
        self.style_err.set_property( "family", "Monospace" )
        self.style_err.set_property( "size_points", 8 )
        self.longtext.get_buffer().get_tag_table().add( self.style_err )
        
        if modal:
            self.dialog.set_modal( True )
        if text != "":
            self.set_text( text )
        if longtext != "" and longtext != None:
            self.set_long_text( longtext )
        
    def set_text( self, text ):
        self.text.set_markup( text )
    
    def set_long_text( self, longtext ):
        buffer = self.longtext.get_buffer()
        start, end = buffer.get_bounds()
        buffer.insert_with_tags( end, longtext, self.style_err )
        
    def run( self ):
        self.dialog.show_all()
        return self.dialog.run()

    def destroy( self ):
        return self.dialog.destroy()  


class ConfimationDialog:
    def __init__( self, parent, xmlfile, action, pkg, size ):
        self.filterTree=gtk.glade.XML ( xmlfile, 'confirmation' )
        self.dialog = self.filterTree.get_widget( "confirmation" )
        #self.win.connect( "destroy", self.quit )
        #self.win.connect( "delete_event", self.quit )
        self.dialog.set_transient_for( parent )
        self.action = self.filterTree.get_widget( "confAction" )
        tit = _( "<span size='x-large'> Packages to Process</span>" )
        self.action.set_markup( tit )
        self.pkg = self.filterTree.get_widget( "confPkg" )
        self.pkgModel = self.setup_view( self.pkg )
        self.show_data( self.pkgModel, pkg )
        self.size = self.filterTree.get_widget( "confSize" )
        self.size.set_text(str(size))
        self.pkg.expand_all()

    def run( self ):
        self.dialog.show_all()
        return self.dialog.run()
             
    def setup_view( self, view ):
        model = gtk.TreeStore( gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING )
        view.set_model( model )
        self.create_text_column( _( "Name" ), view, 0 )
        self.create_text_column( _( "Arch" ), view, 1 )
        self.create_text_column( _( "Ver" ), view, 2 )
        self.create_text_column( _( "Repository" ), view, 3 )
        self.create_text_column( _( "Size" ), view, 4 )
        return model

    def create_text_column( self, hdr, view, colno, min_width=0 ):
         cell = gtk.CellRendererText()    # Size Column
         column = gtk.TreeViewColumn( hdr, cell, markup=colno )
         column.set_resizable( True )
         if not min_width == 0:
             column.set_min_width( min_width )
         view.append_column( column )        
             
             
    def show_data( self, model, pkglist ):
        model.clear()       
        for sub, lvl1 in pkglist:
            label = "<b>%s</b>" % sub
            level1 = model.append( None, [label, "", "", "", ""] )
            for name, arch, ver, repo, size, replaces in lvl1:
                level2 = model.append( level1, [name, arch, ver, repo, size] )
                for r in replaces:
                    level3 = model.append( level2,[ r, "", "", "", ""])

    def run( self ):
        self.dialog.show_all()
        return self.dialog.run()

    def destroy( self ):
        return self.dialog.destroy()
                 
        
class AboutDialog:        
    def __init__( self, parent, version, credits ):
        self.MAIN_PATH = os.path.abspath( os.path.dirname( sys.argv[0] ) );
        self.GLADE_FILE = self.MAIN_PATH+'/yumex.glade'  
        self.xml = gtk.glade.XML( self.GLADE_FILE, "about" )
        self.dialog = self.xml.get_widget( "about" )
        self.parent = parent
        self.dialog.set_transient_for( parent )
        #self.dialog.set_icon_name( 'gtk-dialog-info' )
#        self.dialog.connect( "destroy", self.destroy )
#        self.dialog.connect( "delete_event", self.destroy )
        self.close = self.xml.get_widget( "aboutClose" )
        self.close.connect( "clicked", self.destroy )
        #self.bOK = self.filterTree.get_widget( "aboutOK" )
        #self.bOK.connect( "clicked", self.click, "OK" )
        self.image = self.xml.get_widget( "aboutImage" )
        self.image.set_from_file( "/usr/share/pixmaps/yumex/logo.png" )
        self.image.show()
        self.ver = self.xml.get_widget( "aboutVer" )
        self.ver.set_text( version )
        self.set_longText( credits )
        
        
    def set_longText( self, text ):
        tw = self.xml.get_widget( "aboutCredits" )
        style_err = gtk.TextTag( "stderr" ) 
        style_err.set_property( "style", pango.STYLE_ITALIC )
        style_err.set_property( "foreground", "red" )
        style_err.set_property( "family", "Monospace" )
        style_err.set_property( "size_points", 8 )
        tw.get_buffer().get_tag_table().add( style_err )
        start, end = tw.get_buffer().get_bounds()
        tw.get_buffer().insert_with_tags( end, text, style_err )
        
    def run( self ):
        self.dialog.show()
        return self.dialog.run()

    def destroy( self, widget=None, data=None ):
        return self.dialog.destroy()
               
def about_dialog( parent, version, credits ):
    dlg = AboutDialog( parent, version, credits )
    rc = dlg.run()
    dlg.destroy()

def file_open_box( modal=True, pattern="*.*" ):
    return file_sel_box( "Open", modal=modal, multiple=False, pattern=pattern )

def file_save_box( modal=True, pattern="*.*" ):
    return file_sel_box( "Save As", modal=modal, multiple=False, pattern=pattern )
    
def input_box( title="Input Box", message="", default_text='', 
          modal= True ):
     dlg = EntryDialog( message, default_text, modal=modal )
     dlg.set_title( title )
     dlg.show()
     gtk.main()
     return dlg.ret

def info_box( parent, title, text, text2=None, modal= True,cancel=False ):
     if cancel:
        buttons = gtk.BUTTONS_OK_CANCEL
     else:   
        buttons = gtk.BUTTONS_OK
     dlg = gtk.MessageDialog( parent, gtk.DIALOG_MODAL, 
                              gtk.MESSAGE_INFO, buttons, 
                              text )
     dlg.set_markup( "<b>%s</b>" % text )                     
     if text2:
         dlg.format_secondary_text( text2 )
     rc = dlg.run()
     dlg.destroy()
     return rc == gtk.RESPONSE_OK

def err_box( parent, title, text, longtext=None, modal= True ):
     dlg = ErrorDialog( parent, title, text, longtext, modal )
     rc = dlg.run()
     dlg.destroy()
     
def file_sel_box( title="Browse", modal=False, multiple=True, pattern="*.*" ):
    win = FileChooser( modal=modal, multiple=multiple, pattern=pattern )
    win.set_title( title )
    win.show()
    gtk.main()
    return win.ret     
    
def confimation_dialog( parent, xml, action, pkg, size ):
    dlg = ConfimationDialog( parent, xml, action, pkg, size )
    rc = dlg.run()
    dlg.destroy()
    return rc == gtk.RESPONSE_OK
        