#!/usr/bin/python -tt
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# 
# $Id: yumexPackages.py,v 1.8 2006/02/14 11:02:33 timlau Exp $

#import yumexBase
import re
#import yumexUtils
import time
import gtk

class yumexPackage( object ):
    """ This class contains a yumPackage and some extra features used by
    yumex """
    
    def __init__( self, pkg, recentlimit, avail=True ):
        self.pkg = pkg
        self.selected = False
        self.visible = True
        self.queued = None  
        self.availible = avail
        self.ver = self.pkg.printVer()
        self.summaryFirst = self._toUTF(self.pkg.returnSimple( 'summary' ).splitlines()[0])
        self.size = pkg.size()
        self.sizeKB = "%06d" % ( float( self.size )/ 1024.0 )
        self.time = self._get_time()
        if self.time > recentlimit:
            self.recent = True
        else:
            self.recent = False
 
    def __getattr__(self,attr):
        # Check for attributes, contained in the pkg object 
        if self.availible:
            return self._get_availible_attr(attr)
        else:
            return self._get_installed_attr(attr)

    def _get_availible_attr(self,attr):
        if attr in self.__dict__.keys():       # obj attribute ? 
            return self.__dict__.get(attr)
        elif attr in self.pkg.__dict__.keys(): # pkg attribute ?
           return getattr(self.pkg,attr)        
        elif attr in self.pkg.simple.keys():   # pkg simple dict ?
           return self.pkg.returnSimple(attr)
        else: raise AttributeError, attr
        
    def _get_installed_attr(self,attr):
        if attr in self.__dict__.keys():       # obj attribute ?
            return self.__dict__.get(attr)
        else:
            # we can't test for member so just try to get it.
            try: 
                return self.pkg.returnSimple(attr)
            except KeyError:
               # We want a AttributeError Exception, if not found 
               raise AttributeError, attr
        
            
    def __str__( self ):
        return str( self.pkg )
            
    def get_fullname( self ):
        """ return fullpackage name in format : <epoch>:<name>-<ver>.<arch>"""
        fe = self.ver.find( ":" )
        if fe > -1:
            epoch = self.ver[0:fe]
            ver = self.ver[fe+1:]
            add = "%s:%s-%s.%s" % ( epoch, self.name, ver, self.arch )
        else:
            add = "%s:%s-%s.%s" % ( '0', self.name, self.ver, self.arch )
        return add

    def _toUTF( self, txt ):
        """ this function convert a string to unicode to make gtk happy"""
        rc=""
        try:
            rc = unicode( txt, 'utf-8' )
        except UnicodeDecodeError, e:
            #print e
            #print txt
            rc = unicode( txt, 'iso-8859-1' )
        return rc
    
    def _get_time( self ):
        if not self.availible: # Installed packages dont have filetime
            ftime = int( self.pkg.returnSimple( 'installtime' ) )
        else:
            ftime = int( self.pkg.returnSimple( 'filetime' ) )
        return ftime
    
    def set_select( self, state ):
        self.selected = state

    def set_visible( self, state ):
        self.visible = state

    def get_changelog( self ):    
        """ Get changelog from package object"""
        cl = []
        if self.availible: # YumAvailablePackage
            clog = self.pkg.returnChangelog()
            for tim, name, text in clog:
                cl.append( str( time.ctime( float( tim ) ) )+" "+name+"\n"+text )
        else:  # YumInstalledPackage, get files from rpm header.
            cltime = self.pkg.hdr['changelogtime']
            clname = self.pkg.hdr['changelogname']
            cltext = self.pkg.hdr['changelogtext']
            for tim, name, text in zip( cltime, clname, cltext ):
                cl.append( str( time.ctime( tim ) )+" "+name+"\n"+text )
        return cl
        
    def get_filelist( self ):    
        """ Get filelist from package object"""
        if self.availible: # YumAvailablePackage
            files = self.pkg.returnFileEntries()
        else:  # YumInstalledPackage, get files from rpm header.
            files = []
            fn = self.pkg.hdr['basenames']
            dn = self.pkg.hdr['dirnames']
            for d, n in zip( dn, fn ):
                files.append( d+n )
        return files
        

class yumexPackageList:
    """ This class is a list of yumexPackages """
    
    def __init__( self ):
        self.list  = gtk.ListStore( object )
        self.filter = []
        self.arch_filter = None
        
    def clean( self ):
        self.list.clear()
        self.filter = []
        self.arch_filter = None       

    def dump(self):
        for el in self.list:
            pkg = el[0]
            print pkg.name,pkg.visible

    def add( self, pkg, avail, recentlimit ):
        newpkg = yumexPackage( pkg, recentlimit, avail )
        self.list.append( [newpkg] )
        return newpkg
        
    def add_list( self, pkglist):
        for pkg in pkglist:
            self.list.append( [pkg] )
    
    def set_selection( self, name, ver, arch, state ):
        pkg = self.find_pkg( name, ver, arch )
        if pkg != None:
            if state:
                pkg.select()
            else:
                pkg.unselect()
        
            
    def find_pkg( self, name, ver, arch ):
        rc = None
        for pkg in self.list:
            if pkg[0].name == name:
                v = pkg[0].ver
                if pkg[0].arch == arch and v == ver:
                    rc = pkg[0]
                    break
        return rc
    
    def find_po_by_name( self, n, a ):
        '''  Find package by name,arch and return yum po'''
        rc = None
        for pkg in self.list:
            if n == pkg[0].name:
                if a == pkg[0].arch:
                    rc = pkg[0].pkg
                    break
        return rc

    def find_pkg_by_name( self, n, a ):
        '''  Find package by name,arch and return yumexPackage'''
        rc = None
        for pkg in self.list:
            if n == pkg[0].name:
                if a == pkg[0].arch:
                    rc = pkg[0]
                    break
        return rc

    def find_pkg_by_nva( self, n,v,a ):
        '''  Find package by name,arch and return yumexPackage'''
        rc = None
        for pkg in self.list:
            if n <> pkg[0].name:
                continue
            if a <> pkg[0].arch:
                continue
            if v <> pkg[0].ver:
                continue
            return pkg[0]
        return rc
    
    def find_ver( self, name, arch ):
        rc = None
        for pkg in self.list:
            if pkg[0].name == name:
                if pkg[0].arch == arch:
                    rc = pkg[0].ver
        return rc
    
    def get_active_list( self ):
        return self.list

    def get_selected( self ):
        selected = []
        for pkg in self.list:
            if pkg[0].selected:
                selected.append( pkg[0] )
        return selected

    def get_nonselected( self ):
        nonselected = []
        for pkg in self.list:
            if not pkg[0].selected:
                nonselected.append( pkg[0].pkg )
        return nonselected
        
    def get_lenght( self ):
        return len( self.list )
        
    def select_by_fullname( self, keys ):    
        for pkg in self.list:
            if pkg[0].get_fullname() in keys:
                pkg[0].set_select( True )      
        
    def select_all( self ):
        for pkg in self.list:
            if pkg[0].visible:
                if not pkg[0].selected:
                    pkg[0].set_select( True )      

    def deselect_all( self ):
        for pkg in self.list:
            if pkg[0].visible:
                if pkg[0].selected:
                    pkg[0].set_select( False )      
        
        
    def apply_filter( self, filter, recentFlag=0, selectFlag=None ):
        """
        Apply filters to packages in list.
        All packages there matches the filter will have
        visible=True, else they will have visible=False
        only packages with visible=True, will be shown in the
        Install/Update/Remove TreeViews.
        """
        if filter != None:
            self.filter = filter.split( ' ' )
            self.arch_filter = self._get_arch_filter( filter )
            self.repo_filter = self._get_repo_filter( filter )
        for pkg in self.list:
            if selectFlag and not pkg[0].selected:
                pkg[0].set_visible( False )
            elif recentFlag == 1 and not pkg[0].recent:
                pkg[0].set_visible( False )
            elif recentFlag == 2 and not pkg[0].queued:
                pkg[0].set_visible( False )
            else:
                pkg[0].set_visible( True )
                if len( self.filter ) != 0:
                    self._check_filter( pkg[0] )
                                
    def _check_filter( self, pkg ):
        exit = self._check_special_filters( pkg )
        if exit:
            return
        name = pkg.name
        summary = pkg.summary
        desc = pkg.description
        if name == None: name = ""
        if summary == None: summary = ""
        if desc == None: desc = ""
        keys = [name.upper(), summary.upper(), desc.upper()]
        words = self.filter
        # Check for filter words
        for word in words:
            if word.find( "=" ) <> -1:
                pass
            else:
                if not self._check_for_word( word, keys ):
                    pkg.set_visible( False )
                    return
                
    def _check_special_filters( self, pkg ):
        exit = False
        # Check for Architechture filter
        if pkg.arch == None:
            arch = ""
        else:
            arch = pkg.arch.upper()
        if self.arch_filter != None:
            if arch == self.arch_filter:
                pkg.set_visible( True )
            else:
                pkg.set_visible( False )
                exit = True
                return exit
        # Check for Repository Filter
        repo = pkg.repoid        
        if self.repo_filter != None and repo != "installed":
            if repo.upper() == self.repo_filter:
                pkg.set_visible( True )
            else:
                pkg.set_visible( False )
                exit = True
        return exit
        
                
    def _get_arch_filter( self, flt ):
        """ Check for ARCH=<Actitecture> in filter words """
        arch_re = re.compile( '(?<=ARCH=)\w+' )
        arch_rc = arch_re.search( flt )
        if arch_rc != None:
            #print "Arch : %s" % arch_rc.group( 0 )
            arch_flt = arch_rc.group( 0 )
        else:
            arch_flt = None
        return arch_flt

    def _get_repo_filter( self, flt ):
        """ Check for REPO=<Actitecture> in filter words """
        repo_re = re.compile( '(?<=REPO=)\w+' )
        repo_rc = repo_re.search( flt )
        if repo_rc != None:
            #print "Repo : %s" % repo_rc.group( 0 )
            repo_flt = repo_rc.group( 0 )
        else:
            repo_flt = None
        return repo_flt

            
    def _check_for_word( self, word, keys ):
        for key in keys:
            if key.find( word ) <> -1:
                return True
        return False
        
