#!/usr/bin/python -tt
# -*- coding: iso-8859-1 -*-
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# 
# $Id: yumexmain.py,v 1.49 2006/02/19 05:41:04 timlau Exp $

import sys
import os
import gtk
import gtk.glade
import gobject
#import ConfigParser
import re
import time
import thread


from types import *

import yumex.yumexConst as const
from yumex.yum24XBase import *
from yumex import *
from yumex.yumexi18n import _
from optparse import OptionParser
import locale

try:
    from rhpl.exception import installExceptionHandler
    EXCPECTION_HANDLER = True
except:
    EXCPECTION_HANDLER = False
    
   
from yumgui import TextViewLogger
from yumgui.widgets import UI,Controller
import yum

class YumexController( Controller ):
    ''' This class contains all glade signal callbacks '''
    
    def __init__( self ):
         # Create and ui object contains the widgets.
         ui = UI(const.GLADE_FILE , 'main','yumex' )
         # init the Controller Class to connect signals.
         Controller.__init__(self, ui)
        
    def on_toolbar( self, widget, page ):
        ''' Left Side Toolbar Handler'''
        if widget.get_active():
            self.activePage = page
            if page in ['update','install','remove']:
                self.pkgNotebookClear()
                self.packages.set_page(page)
                self.doFilter()
            self.ui.notebook.set_current_page(const.NOTEBOOK_PAGES[page])


    
    def on_view_cursor_changed( self, widget ):
        """ Handle selection of row in install/update/remove view (Show Descriptions) """
        ( model, iterator ) = widget.get_selection().get_selected()
        if model != None and iterator != None:
            self.pkgNotebookClear()
            name = model.get_value( iterator, 1 )
            ver = model.get_value( iterator, 2 )
            arch = model.get_value( iterator, 5 )
            pkg = self.packages.get_packageList().find_pkg( name, ver, arch )
            if pkg != None:
                self.writePkg(self.pkgDesc,pkg,"%s","description")
                if self.check_option( "filelists" ) or pkg.repoid == 'installed':
                    files = pkg.get_filelist()
                    for f in files:
                        self.pkgFiles.write_line("%s\n" % f ) 
                    cl = pkg.get_changelog()
                    for l in cl:
                        self.pkgChangeLog.write_line( "%s\n" % l ) 
                if self.activePage=='update':
                    ver = self.find_current_installed( name, arch )
                    txt = _( "Installed version: %s (%s) \n\n" ) % ( ver, arch )
                    self.pkgInfo.write_line(txt ) 
                self.writePkg(self.pkgInfo,pkg,'Group        : %s\n',"group",True)
                self.writePkg(self.pkgInfo,pkg,'Source       : %s\n',"sourcerpm")
                self.writePkgTime(self.pkgInfo,pkg,'Build Time   : %s\n',"buildtime")
                if self.activePage=='remove':
                    self.writePkgTime(self.pkgInfo,pkg,'Install Time : %s\n',"installtime")
                    self.writePkg(self.pkgInfo,pkg,'License      : %s\n','license')
                    self.pkgOther.write_line( _("Requires:\n\n")) 
                    reqList = pkg.pkg.requiresList()
                    for req in reqList:
                        self.pkgOther.write_line( "%s\n" % req )                     
                self.pkgNotebookGoTop
                        
                        
    def on_AddQueue_clicked( self, widget ):
        """ Add to Queue Button Handler """
        pkgs = self.packages.curView.get_selected()
        action = self.activePage[0].lower()
        for po in pkgs: 
            self.queueView.add( po, action )
            po.queued = action
            po.set_select( False )
            self.queueView.populate_view()
        self.packages.view.queue_draw()
        self.set_status(_('%i packages added to queue') % len(pkgs))

    def on_queueProcess_clicked( self, widget ):
        """ Process Queue Button Handler """
        yumgui.busyCursor(self.ui.main)
        self.pageSave = self.activePage
        doAll = self.check_update_all()
        pkgs = self.queueView.get()
        if self.action.process_package_queue( pkgs, doAll):
            self.set_page( self.pageSave )                
            yumgui.normalCursor(self.ui.main)
            self.setup_yum()
            self.queueView.clear()
        else:
            yumgui.normalCursor(self.ui.main)
            self.yumexbase.reset_ts()
        self.set_page( self.pageSave )

    def on_queueProcessSafe_clicked( self, widget ):
        """ Process Queue Button Handler """
        yumgui.busyCursor(self.ui.main)
        self.pageSave = self.activePage
        doAll = self.check_update_all()
        pkgs = self.queueView.get()
        # Use SafeMode
        if self.action.process_package_queue( pkgs, doAll,True):
            self.set_page( self.pageSave )                
            yumgui.normalCursor(self.ui.main)
            self.setup_yum()
            self.queueView.clear()
        else:
            yumgui.normalCursor(self.ui.main)
            self.yumexbase.reset_ts()
        self.set_page( self.pageSave )


    def on_queueDel_clicked( self, widget ):
        """ Delete from Queue Button Handler """
        self.queueView.deleteSelected()

    def on_Select_clicked( self, widget ):
        """ Select All Button Handler """
        yumgui.busyCursor(self.ui.main)        
        self.packages.curView.select_all()
        yumgui.normalCursor(self.ui.main)
            
    def on_deSelect_clicked( self, widget ):
        """ DeSelect All Button Handler """
        yumgui.busyCursor(self.ui.main)        
        self.packages.curView.deselect_all()
        yumgui.normalCursor(self.ui.main)
            
    def on_repoRefresh_clicked( self, widget ):
        """ Repo Refresh Button Handler """
        self.setup_yum()            

    def on_repoDeSelect_clicked( self, widget ):
        """ Repo DeSelect All Button Handler """
        self.repoView.deselect_all()
             

    def on_search_clicked( self, widget ):
        """ Search Button Handler """
        yumgui.busyCursor(self.ui.main)        
        self.doFilter()            
        yumgui.normalCursor(self.ui.main)

    def on_clear_clicked( self, widget ):
        """ Clear button handler """
        yumgui.busyCursor(self.ui.main)        
        self.resetFilters()
        self.doFilter()
        yumgui.normalCursor(self.ui.main)

    def on_forceArch_toggled( self, widget ):
        """ Force i386 Checkbox Handler """
        self.forceArch = not self.forceArch
        if self.forceArch:
            self.set_status( _( "Forcing i386 architecture for $basearch variable" ) )
        else:
            self.set_status( _( "Using system's $basearch variable" ) )
    
    
                
# Menu Handlers                  
                    
    def on_FileQuit( self, widget ):
        self.delete_event()
        
    def on_EditPreferences( self, widget ):
        self.doPref()           
        
    def on_HelpAbout( self, widget ):
        ver = "Yum Extender %s" % ( const.__yumex_version__ )
        about_dialog( self.ui.main, ver, const.__credits__ )

    def on_ProfileSave( self, widget ):
        self.save_profile()
            
    def on_ProfileNew( self, widget ):
        self.new_profile()

    def on_ToolsRepoCache( self, widget ):
        if self.yumexbase != None:
            self.yumexbase.repos_in_cache.repos = {}
            rc, msg = self.yumexbase.cleanMetadata()
            self.yumexbase.log( 2, msg[0] )       
            self.set_status( msg[0] )        
            self.yumexbase.repos_in_cache.clear()
                
# Repository View Handlers

    def on_viewRepo_button_press_event( self, treeview, event ):
        ''' Repository View Right Click Handler'''
        if event.button == 3: # Right Click
            x = int( event.x )
            y = int( event.y )
            time = event.time
            pthinfo = treeview.get_path_at_pos( x, y )
            if pthinfo != None:
                path, col, cellx, celly = pthinfo
                treeview.grab_focus()
                treeview.set_cursor( path, col, 0 )
                self.repo_popup.popup( None, None, None, event.button, time )
                
    def on_repo_popup( self, label ):
        ''' Repository Right Click Menu Handler '''
        ( model, iterator ) = self.ui.viewRepo.get_selection().get_selected()       
        if model != None and iterator != None:
            activeRepo = model.get_value( iterator, 1 )
            if label == _( "_New" ):
                editrepo = EditRepo( const.GLADE_FILE, activeRepo, isNew=True )
                self.repo.update()
                self.setup_repoview()
                self.on_profile( self.profile.get_active() )                
            elif label == _( "_Edit" ):
                editrepo = EditRepo( const.GLADE_FILE, activeRepo )
            elif label == _( "_Delete" ):
                text = _( " Do you want to delete the <b>%s</b> repository?" ) % activeRepo
                rc = info_box( self.main, _( "Delete Repository" ), text, cancel=True )
                if rc:
                    self.repo.delete_repo( activeRepo )
                    self.setup_repoview()
                    self.on_profile( self.profile.get_active() )                    
                
    def init_repo_popup( self ):
        ''' Create Repository Right Click Menu '''
        popup_menu = gtk.Menu()
        self.add_to_menu( popup_menu, _( "_New" ), self.on_repo_popup )
        self.add_to_menu( popup_menu, _( "_Edit" ), self.on_repo_popup )
        self.add_to_menu( popup_menu, _( "_Delete" ), self.on_repo_popup )
        return popup_menu

# Group View Handlers

    def on_viewGroups_cursor_changed( self, widget ):
        """ Handle selection of row in group view"""
        data = []
        ( model, iterator ) = widget.get_selection().get_selected()
        if model != None and iterator != None: 
            path = model.get_path( iterator )
            row = model[path]
            if row.parent != None:
                grp = row[0]
                data = self.getGroupData( grp )
                manData = []
                defData = []
                optData = []
                if data.has_key('mandatory'):
                    manData = data['mandatory']
                if data.has_key('default'):
                    defData = data['default']
                if data.has_key('optional'):
                    optData = data['optional']    
                self.grpInfoMan.populate_view(manData )
                self.grpInfoDef.populate_view(defData )
                self.grpInfoOpt.populate_view(optData )


    def on_viewGroups_button_press_event( self, treeview, event ):
        ''' Group View RightClick Handler'''
        if event.button == 3: # Right Click
            x = int( event.x )
            y = int( event.y )
            time = event.time
            pthinfo = treeview.get_path_at_pos( x, y )
            if pthinfo != None:
                path, col, cellx, celly = pthinfo
                treeview.grab_focus()
                treeview.set_cursor( path, col, 0 )
                ( model, iterator ) = self.ui.viewGroups.get_selection().get_selected()       
                if model != None and iterator != None:
                    # Get Section number
                    installed = model.get_value( iterator, 2 )
                    grp_menu = gtk.Menu()
                    self.add_to_menu( grp_menu, _( "_Add to Queue" ), self.on_group_popup )
                    grp_menu.popup( None, None, None, event.button, time )

    def on_viewGroupInfo_button_press_event( self, treeview, event ):
        ''' Group View RightClick Handler'''
        if event.button == 3: # Right Click
            x = int( event.x )
            y = int( event.y )
            time = event.time
            pthinfo = treeview.get_path_at_pos( x, y )
            if pthinfo != None:
                path, col, cellx, celly = pthinfo
                treeview.grab_focus()
                treeview.set_cursor( path, col, 0 )
                ( model, iterator ) = self.ui.viewGroups.get_selection().get_selected()       
                if model != None and iterator != None:
                    # Get Section number
                    menu = gtk.Menu()
                    self.add_to_menu( menu, _( "_Add to Queue"  ), self.on_groupinfo_popup )
                    menu.popup( None, None, None, event.button, time )

    def on_groupinfo_popup( self, label ):
        ''' Group View Right Click Menu Handler '''
        page = self.ui.groupNotebook.get_current_page()
        if page == 0:
            view = self.grpInfoMan
        elif page == 1:
            view = self.grpInfoDef
        else:
            view = self.grpInfoOpt
        ( model, iterator ) = view.view.get_selection().get_selected()       
        if model != None and iterator != None:
            path = model.get_string_from_iter(iterator)
            row = view.curView.get_child_path_row( path )
            pkg = row[0]
            if not pkg.availible:
                self.queueView.add( pkg, 'r')
                pkg.queued = 'r'
                self.queueView.populate_view()
                msg = _("added remove of %s to Queue") % str(pkg)
                self.yumexbase.log(2,msg)
                self.set_status(msg)
            else:
                self.queueView.add( pkg, 'i')
                pkg.queued = 'i'
                self.queueView.populate_view()
                msg = _("added install of %s to Queue") % str(pkg)
                self.yumexbase.log(2,msg)
                self.set_status(msg)
                        
                



    def on_group_popup( self, label ):
        ''' Group View Right Click Menu Handler '''
        ( model, iterator ) = self.ui.viewGroups.get_selection().get_selected()       
        if model != None and iterator != None:
            grp = model.get_value( iterator, 0 )
            state = model.get_value( iterator, 2 )
            pkgs = self.yumexbase.getGroupPackages(grp)
            no = 0
            if state == 1: # installed
                msg = _("%i packages added to queue for removal")
                for name in pkgs:
                    po,inst = self.yumexbase._get_pkg(name)
                    if len(po) > 0 and inst:
                        pkg = self.packages.get_packageList('remove').find_pkg_by_name(po[0].name,po[0].arch)
                        self.queueView.add( pkg, 'r')
                        pkg.queued = 'r'
                        no += 1
                    else:
                        self.yumexbase.errorlog(2,_("package %s was not found") % name)
            elif state == 0: # not installed
                msg = _("%i packages added to queue for installation")
                for name in pkgs:
                    po,inst = self.yumexbase._get_pkg(name)
                    if len(po) > 0:
                        if not inst:
                            pkg = self.packages.get_packageList('install').find_pkg_by_name(po[0].name,po[0].arch)
                            self.queueView.add( pkg, 'i')
                            pkg.queued = 'i'
                            no += 1
                        else:
                            pass # package already installed
                    else:
                        self.yumexbase.errorlog(2,_("package %s was not found") % name)
            model.set_value( iterator, 2, 2 ) # Set state to applied
            self.queueView.populate_view()
            self.yumexbase.log(2,msg % no)
            self.set_status(msg % no)
                    

                    
    def init_group_menu( self ):
        ''' Create Group View Right Click Menu '''
        grp_menu = gtk.Menu()
        self.add_to_menu( grp_menu, _( "_Add to Queue" ), self.on_group_popup )
        
# Profile Menu Handler                    

    def on_profile( self, data ):
        """ Profiles Menu Handler """
        self.set_status( _( "Selected the %s profile" ) % data )
        if data.lower() == "yum-enabled":
            keys = self.repo.get_active_repos()
        else:
            keys = self.profile.get_profile( data )
        if keys != False:
            self.repoView.select_by_keys( keys, False )
            self.set_main_title( data )
            self.activeProfile = data
        else:
            self.set_status( _( "Profile %s not found" ) % data )
            

class YumexMainApplication( YumexController):

    def __init__( self ):
        YumexController.__init__( self )
        self.cmdline_options()
        if self.cmd_options.version:
            print "Yum Extender %s" % ( const.__yumex_version__ )
            sys.exit( 1 )
        self.repoEnabled = []
        self.repoDisabled = []
        self.base_row = -1
        self.updates = -1
        self.lastFilter = {'update':"",'install':"",'remove':""}
        locale.setlocale( locale.LC_ALL, "" )
        gtk.glade.bindtextdomain( 'yumex', '/usr/share/locale' )
        gtk.glade.textdomain ( 'yumex' )
        self.repo = Repository()
        self.profile = Profile()
        self.settings = read_yumex_conf()
        self.update_settings()
        self.debugMode = self.check_option( 'debug' )
        self.yumexbase = None
        self.activeProfile = None
        self.profileGroup = None
        self.forceArch = False
        self.currentAction = None
        
	
        # Set limit for recent packages
        rdays = self.settings.recentdays
        recentdays = float( rdays )
        now = time.time()
        self.recentlimit = now-( recentdays*const.DAY_IN_SECONDS )        
        self.setup_gui()
        yumgui.doGtkEvents()
        # Setup Action handler
        self.action = yumexAction( self.ui.main, const.GLADE_FILE, self.progress, self.logger, self.ui.status )
        self.isRunning = False
        self.StopThread = False
        # Get Yum data
        if self.settings.autorefresh:
            self.setup_yum()            
            
    def cmdline_options( self ):
        """ Handle commmand line options """    
        parser = OptionParser()
        parser.add_option( "-d", "--debug", 
                        action="store_true", dest="debug", default=False, 
                        help="Debug mode" )
        parser.add_option( "-n", "--noauto", 
                        action="store_true", dest="noauto", default=False, 
                        help="No automatic refresh af program start" )
        parser.add_option( "-p", "--noplugins", 
                        action="store_true", dest="noplugins", default=False, 
                        help="Disable yum plugins" )
        parser.add_option( "-C", "--usecache", 
                        action="store_true", dest="usecache", default=False, 
                        help="Use Yum cache only, dont update metadata" )
        parser.add_option( "-O", "--fullobsoletion", 
                        action="store_true", dest="fullobsoletion", default=False, 
                        help="Do full obsoletion every time" )
        parser.add_option( "-v", "--version", 
                        action="store_true", dest="version", default=False, 
                        help="Show Yum Extender Version" )

        ( options, args ) = parser.parse_args()
        self.cmd_options = options
        self.cmd_args = args

    def update_settings( self ):
        """ update setting with commandline options """
        options = ['noplugins', 'debug', 'usecache', 'fullobsoletion']
        for opt in options:
            if getattr( self.cmd_options, opt ):
                setattr( self.settings, opt, True )
        if self.cmd_options.noauto:
            self.settings.autorefresh = False

    def check_option( self, option ):
        """ Check options in settings or command line"""
        rc = False
        if option == "debug":
            if self.settings.debug or self.cmd_options.debug:
                rc = True
        elif option == "autorefresh":
            if self.settings.autorefresh and not self.cmd_options.noauto:
                rc = True
        elif option == "filelists":
            if self.settings.filelists:
                rc = True
        elif option == "autocleanup":
            if self.settings.autocleanup:
                rc = True
        elif option == "noplugins":
            if self.settings.noplugins or self.cmd_options.noplugins:
                rc = True        
        elif option == "usecache":
            if self.settings.usecache or self.cmd_options.usecache:
                rc = True        
        return rc        
    
    def pkgNotebookClear(self):
        self.pkgDesc.clear()
        self.pkgInfo.clear()
        self.pkgFiles.clear()
        self.pkgChangeLog.clear()
        self.pkgOther.clear()

    def pkgNotebookGoTop(self):
        self.pkgDesc.goTop()
        self.pkgInfo.goTop()
        self.pkgFiles.goTop()
        self.pkgChangeLog.goTop()
        self.pkgOther.goTop()
                
            
    def check_update_all( self ):
        """ Check if all updates are added to the package queue"""
        no = self.packages.curView.packageList.get_lenght()
        pkgs = self.queueView.get( 'u' )
        if len( pkgs ) == no and len( pkgs ) > 0:
            doAll = True
        else:
            doAll = False
        return doAll

    def delete_event( self, widget=None, event=None, data=None ):    
        if self.activeProfile != None:
            self.profile.set_active( self.activeProfile )
        gtk.main_quit()
        sys.exit( 0 )         # Terminate Program

        return False
       
    def save_profile( self ):
        if self.activeProfile != None:
            repos = self.repoView.get_selected()
            if self.profile.set_profile( self.activeProfile, repos ):
                self.set_status( _( "Profile : %s saved ok" ) % self.activeProfile )
            else:
                self.set_status( _( "Profile : %s save failed" ) % self.activeProfile )
        else:
            self.set_status( _( "No profile selected" ) )

    def new_profile( self ):
        name = input_box( _( "Create New Profile" ), _( "_Profile name:" ) )
        if name != None:
            repos = self.get_selected( 0 )
            if self.profile.add_profile( name, repos ):
                self.add_to_profile_menu( name )
                self.activeProfile = name
                self.set_main_title( name )
                self.set_status( _( "Profile : %s created" ) % name )
            else:
                self.set_status( _( "Profile : %s creation failed" ) % name )

    def doPref( self ):
        Preferences( const.GLADE_FILE )
        self.settings = read_yumex_conf()
        self.update_settings()
        self.yumexbase.yumexconf = self.settings
        self.ui.queueProcessSafe.set_property( 'visible', self.settings.processsafemode)
        
    
    def doFilter( self ):
        filter = self.ui.pkgFilter.get_text()
        if filter != self.lastFilter[self.activePage]:
            self.lastFilter[self.activePage] =  filter
            self.packages.curView.apply_filter( filter.upper() )

    def get_notselected( self ):
        notselected = self.repoView.get_notselected()          
        return notselected
        
    def set_main_title( self, extra = None ):
        if extra == None:
            self.ui.main.set_title( "Yum Extender %s" % const.__yumex_version__ )
        else:
            self.ui.main.set_title( "Yum Extender %s - %s Profile" % ( const.__yumex_version__, extra ) )
        
    def add_to_menu( self, menu, label, handler ):
        item = gtk.MenuItem( label )
        item.show()
        menu.append( item )
        item.connect_object( "activate", handler, label )
            
    def add_to_radiomenu( self, menu, label, handler ):
        if self.profileGroup == None:
            item = gtk.RadioMenuItem( None, label, False )
            self.profileGroup = item
        else:
            item = gtk.RadioMenuItem( self.profileGroup, label, False )
        if self.profile.get_active() == label:
            item.set_active( True )
        item.show()
        menu.append( item )
        item.connect_object( "activate", handler, label )

    def set_page( self, page ):
        self.activePage = page
        widget = getattr( self,  const.PAGE_BUTTONS[page] )
        widget.set_active( True )
        
    
    def setup_gui( self ):
        """ Setup GUI """
        gtk.window_set_default_icon_from_file( const.PIXMAPS_PATH + '/yumex-icon.png' )
        self.ui.main.set_icon_from_file( const.PIXMAPS_PATH + '/yumex-icon.png' )
        self.ui.main.connect( "delete_event", self.delete_event )
        self.set_main_title()
        self.ui.main.present()
        # Setup Vertical Toolbar
        self.toolButton0 = self.createButton( _( "Update" ), "button-update.png", None, 'update' )
        self.toolButton1 = self.createButton( _( "Install" ), "button-install.png", self.toolButton0, 'install' )
        self.toolButton2 = self.createButton( _( "Remove" ), "button-remove.png", self.toolButton0, 'remove' )
        self.toolButton3 = self.createButton( _( "Queue" ), "button-group.png", self.toolButton0, 'queue' )
        self.toolButton4 = self.createButton( _( "Repos" ), "button-repo.png", self.toolButton0, 'repos' )
        self.toolButton5 = self.createButton( _( "Output" ), "button-output.png", self.toolButton0, 'output')
        self.toolButton6 = self.createButton( _( "Groups" ), "button-group.png", self.toolButton0, 'group' )
        style = self.ui.leftEvent.get_style()
        bgcolor = gtk.gdk.Color( 65535, 65535, 65535 )
        self.ui.leftEvent.modify_bg( gtk.STATE_NORMAL, bgcolor )
        self.ui.pageImage0.set_from_file ( const.PIXMAPS_PATH + '/button-repo.png' )
        self.ui.pageImage2.set_from_file ( const.PIXMAPS_PATH + '/button-output.png' )
        self.ui.pageImage3.set_from_file ( const.PIXMAPS_PATH + '/button-group.png' )
        self.ui.pageImage4.set_from_file ( const.PIXMAPS_PATH + '/button-group.png' )
        # Setup Notebook
        self.ui.notebook.set_show_tabs( False )
        # Show force i386 checkbox only for x86_64 platforms
        if not os.uname()[4] == "x86_64": 
            self.ui.forceArchOption.hide()
        # Setup filter widget
        #self.pkgFilter.connect( "activate", self.on_search_clicked )
        self.ui.pkgFilter.set_text( "" )
        # Set Description TextViews
        self.pkgDesc = TextViewConsole(self.ui.pkgDesc)
        self.pkgInfo = TextViewConsole(self.ui.pkgInfo)
        self.pkgFiles = TextViewConsole(self.ui.pkgFiles)
        self.pkgChangeLog = TextViewConsole(self.ui.pkgCLog)
        self.pkgOther = TextViewConsole(self.ui.pkgOther)
        # Add Profiles to Profiles menu.        
        self.add_to_profile_menu( 'yum-enabled' )
        for p in self.profile.get_list():
            self.add_to_profile_menu( p )
        # Init progress dialog  
        self.progress = ProgressDialog( self.ui, const.PIXMAPS_PATH, self.ui.notebook, self.set_page )
        # Repository View
        headers = [_( "Repository" ), _( "File" )]
        self.repoView = repoView( self.ui.viewRepo, headers )
        self.repo_popup = self.init_repo_popup()
        # Package View
        self.packages = Packages(self.ui, self.ui.viewPkg, self.progress)
        self.set_page( 'update' )
        # Group View
        self.grpView = grpView( self.ui.viewGroups )
        self.grpInfoMan = grpInfoView( self.ui.viewGroupMan, self.progress )
        self.grpInfoDef = grpInfoView( self.ui.viewGroupDef, self.progress )
        self.grpInfoOpt = grpInfoView( self.ui.viewGroupOpt, self.progress )
        # Queue View
        self.queueView = queueView( self.ui.queueView )
        # logger
        self.logger = TextViewLogger( self.ui.viewOutput, addtime=self.settings.debug )
        self.setup_repoview()
        # Show/hide Process Queue (SafeMode)
        self.ui.queueProcessSafe.set_property( 'visible', self.settings.processsafemode)
        # Reload last profile
        self.on_profile( self.profile.get_active() )


    def resetFilters( self ):
        self.ui.pkgFilter.set_text( "" )

    def createButton( self, text, icon, group, page ):
          button = gtk.RadioButton( group )
          button.connect( "clicked", self.on_toolbar, page )

          button.set_relief( gtk.RELIEF_NONE )
          button.set_mode( False )

          p = gtk.gdk.pixbuf_new_from_file( const.PIXMAPS_PATH+"/"+icon )
          pix = gtk.Image()
          pix.set_from_pixbuf( p )
          pix.show()

          buttonLabel = gtk.Label( "" )
          buttonLabel.set_markup( '<span foreground="#000000">%s</span>' % text )
          buttonLabel.show()
          buttonVBox = gtk.VBox()
          buttonVBox.pack_start( pix )
          buttonVBox.pack_start( buttonLabel )
          button.add( buttonVBox )
          button.show()
          buttonVBox.show()
          self.ui.content.pack_start( button, False )
          return button

    def add_to_profile_menu( self, name ):
        sub = self.ui.profileMenu.get_submenu()
        self.add_to_radiomenu( sub, name, self.on_profile )
    
    def set_status( self, text ):
        context_id = self.ui.status.get_context_id( "Status" )
        self.ui.status.push( context_id, text )
   

    def setup_combo_entry( self, entry ):
        model = gtk.ListStore( gobject.TYPE_STRING )
        entry.set_model( model )
        return model

    def setup_repoview( self ):
        repos = self.repo.get_names_sorted()
        data = []
        i = 0
        for r in repos:
            if r == 'base':
                self.base_row = i
            if r == 'updates-released':
                self.updates_row = i      
            el = ( False, r, self.repo.get_repo_filename( r ) )
            data = data + [el]
            i = i+1 
        self.repoView.populate_view( data )

    def find_current_installed( self, name, arch ):
        return self.packages.findVer( name, arch,'remove' )

    def getGroupData(self,name):
        ypdict = {}
        data = self.yumexbase.getGroupData( name )
        for key in data.keys():
            pkgs = data[key]
            ypkgs = []
            for po,inst in pkgs:
                name = po.name
                ver = po.printVer()
                arch = po.arch
                if inst:
                    yp = self.packages.findPkg(name,ver,arch,'remove')
                else:
                    yp = self.packages.findPkg(name,ver,arch,'install')
                if yp:
                    yp.set_select(inst)
                    ypkgs.append(yp)
            ypdict[key] = ypkgs
        return ypdict

    def update_page( self, no, update=False ):
        if no == "GROUPS":
            self.yumexbase.progresslog( _( "Updating %s view, populate view with data " ) % no.lower() )
            if yum.__version__ < '2.5.0':
                self.grpView.populate_view_yum24( self.yumexbase.groupInfo )
            else:
                self.grpView.populate_view( self.yumexbase.comps )
        self.set_status( "" )

    def toUTF( self, txt ):
        rc=""
        try:
            rc = unicode( txt, 'utf-8' )
        except UnicodeDecodeError, e:
            rc = unicode( txt, 'iso-8859-1' )
        return rc
        
    def check_proxy( self ):
        proxy = self.settings.proxy
        if proxy:
            self.yumexbase.log( 2, _( "Using http_proxy=%s" ) % proxy ) 
            os.environ["http_proxy"]=proxy

    def writePkg(self,outobj,pkg,markup,attr,remove_newline=False):
        try:
            data = getattr(pkg,attr)
            if remove_newline:
                out = markup % data.replace("\n","")
            else:
                out = markup % data 
            outobj.write_line(out)
        except AttributeError,e:
            msg = _('Can not read the %s attribute') % attr
            print msg

    def writePkgTime(self,outobj,pkg,markup,attr):
        try:
            data = getattr(pkg,attr)
            data = str( time.ctime( float( data ) ) )
            out = markup % data 
            outobj.write_line(out)
        except AttributeError,e:
            msg = _('Can not read the %s attribute') % attr
            print msg
         
    def setup_yum( self):
        ''' Setup yum and read repository metadata and populate views '''
        self.currentAction = 0
        t = time.time()
        self.pageSave = self.activePage
        self.progress.show()  
        self.progress.set_mainLabel( _( "Initializing Repositories" ) )        
        loglevel = 2
        if self.yumexbase == None:
            if yum.__version__ < '2.5.0':
                self.yumexbase = yumexBase24( self.settings, self.logger, self.progress, loglevel )
            else:
                self.yumexbase = yumexBase( self.settings, self.logger, self.progress, loglevel )
            self.yumexbase.forceArch = self.forceArch
            self.action.set_yumbase( self.yumexbase )
            rload = False
        else:
            rload = True
            self.yumexbase.forceArch = self.forceArch
            self.yumexbase.cleanup()
        self.check_proxy()
        yumgui.busyCursor(self.ui.main)
        self.progress.total.setup( const.REPOSETUP_PROGRESS_STEPS )
        if self._runBackground( self.yumexbase.setup, rload ):
            self._runBackground( self.yumexbase.set_active_repos, self.repoView.get_selected() )
            self.progress.total.next()
            if self._runBackground( self.yumexbase.load_repo_data, rload ):
                pkglists = self.packages.get_packageViews()
                self.progress.total.next()
                self.yumexbase.build_package_lists( self.recentlimit, pkglists )
                self.update_page( "GROUPS" )
                self.progress.hide()  
                endtime = time.time()-t
                msg = _( "Repository initialization completed in %.2f seconds" ) % ( endtime )
                self.set_status( msg )
                self.yumexbase.log( 2, msg )
                self.resetFilters()     
                self.queueView.clear()   # Clear Queue   
                self.pkgNotebookClear()
                self.set_page( self.pageSave )
                #self.testing_something()
                yumgui.normalCursor(self.ui.main)
            else:
                yumgui.normalCursor(self.ui.main)
                self.progress.hide()
                msg = _( "Error in loading repository data" )
                yumexDialogs.err_box( self.ui.main, _( "Error" ), msg, self.yumexbase.last_error )
        else:
            yumgui.normalCursor(self.ui.main)
            self.progress.hide()
            msg = _( "Error in yum setup" )
            yumexDialogs.err_box( self.ui.main, _( "Error" ), msg, self.yumexbase.last_error )
        self.progress.total.clear()
        self.currentAction = None

    def testing_something( self ):
        for po in self.yumexbase.findPackages( best=True, inst=False, name='yumex' ):
            print "%s - %s" % ( po.compactPrint() , po.repoid )
        
           
    def _runBackground( self, runfunc, *args ):
        """ Start a function in the background """
        self.isRunning = True
        self.backgroundThread = BackgroundRunner( runfunc, args )
        self.backgroundThread.start()
        while self.backgroundThread.isAlive():
            while gtk.events_pending():      # process gtk events
                gtk.main_iteration()        
            time.sleep( 0.1 )
        rc = self.backgroundThread.rc
        del self.backgroundThread
        self.isRunning = False
        return rc
      
if __name__ == "__main__":
    if EXCPECTION_HANDLER:
        installExceptionHandler( "yumex", const.__yumex_version__ )
    mainApp = YumexMainApplication()
    gtk.main()



